# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from django.conf.urls import url

from private_messages.views import InboxView, MessageView, SendMessageView


urlpatterns = [
    url(r'^messages/$', InboxView.as_view(), name='inbox'),
    url(r'^messages/(?P<pk>\d+)/$', MessageView.as_view(), name='read_message'),
    url(r'^messages/new/$', SendMessageView.as_view(), name='send_message'),
]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django import forms
from django.utils.translation import ugettext_lazy as _

from pybb import defaults, util
from private_messages.models import PrivateMessage


class MessageForm(forms.ModelForm):

    class Meta(object):
        model = PrivateMessage
        fields = ('receivers', 'subject', 'body',)
        widgets = {
            'body': util.get_markup_engine().get_widget_cls(),
        }
        labels = {
            'receivers': _('To'),
        }

    def __init__(self, *args, **kwargs):
        super(MessageForm, self).__init__(*args, **kwargs)
        self.available_smiles = defaults.PYBB_SMILES
        self.smiles_prefix = defaults.PYBB_SMILES_PREFIX



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='MessageHandler',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('read', models.BooleanField(default=False)),
                ('deleted', models.BooleanField(default=False)),
            ],
        ),
        migrations.CreateModel(
            name='PrivateMessage',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('body', models.TextField(verbose_name='Message')),
                ('body_html', models.TextField(verbose_name='HTML version')),
                ('body_text', models.TextField(verbose_name='Text version')),
                ('sender_ip', models.IPAddressField(default='0.0.0.0', verbose_name='Sender IP', blank=True)),
                ('sent', models.DateTimeField(auto_now_add=True, verbose_name='Sent', db_index=True)),
                ('subject', models.CharField(default='[No Subject]', max_length=100, blank=True)),
                ('sender_deleted', models.BooleanField(default=False)),
                ('receivers', models.ManyToManyField(related_name='inbox', verbose_name='Recipients', through='private_messages.MessageHandler', to=settings.AUTH_USER_MODEL)),
                ('sender', models.ForeignKey(related_name='outbox', verbose_name='Sender', to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'verbose_name': 'Private Message',
                'verbose_name_plural': 'Private Messages',
            },
        ),
        migrations.AddField(
            model_name='messagehandler',
            name='message',
            field=models.ForeignKey(to='private_messages.PrivateMessage'),
        ),
        migrations.AddField(
            model_name='messagehandler',
            name='receiver',
            field=models.ForeignKey(to=settings.AUTH_USER_MODEL),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.contrib.auth.decorators import login_required
from django.core.exceptions import PermissionDenied
from django.http import HttpResponseRedirect
from django.utils.decorators import method_decorator
from django.views import generic
from django.views.decorators.vary import vary_on_cookie

from pybb import defaults
from pybb.views import PaginatorMixin

from private_messages.forms import MessageForm
from private_messages.models import PrivateMessage, MessageHandler


class InboxView(PaginatorMixin, generic.ListView):
    paginate_by = defaults.PYBB_TOPIC_PAGE_SIZE
    context_object_name = 'message_list'
    template_name = 'pybb/private_messages/inbox.html'

    def get_queryset(self):
        return self.request.user.inbox.all().select_related()

    @method_decorator(login_required)
    @method_decorator(vary_on_cookie)
    def dispatch(self, request, *args, **kwargs):
        return super(InboxView, self).dispatch(request, *args, **kwargs)


class MessageView(generic.DetailView):

    queryset = PrivateMessage.objects.all()
    template_name = 'pybb/private_messages/message.html'
    context_object_name = 'message'
    http_method_names = ['get', ]

    @method_decorator(login_required)
    @method_decorator(vary_on_cookie)
    def dispatch(self, request, *args, **kwargs):
        return super(MessageView, self).dispatch(request, *args, **kwargs)

    def get_object(self, queryset=None):
        message = super(MessageView, self).get_object(queryset)
        if message not in self.request.user.inbox.all() and message.sender != self.request.user:
            raise PermissionDenied
        try:
            handler = MessageHandler.objects.get(message=message, receiver=self.request.user)
            handler.read = True
            handler.save()
        except MessageHandler.DoesNotExist:
            pass
        return message


class SendMessageView(generic.CreateView):

    template_name = 'pybb/private_messages/new.html'
    form_class = MessageForm

    @method_decorator(login_required)
    def dispatch(self, request, *args, **kwargs):
        return super(SendMessageView, self).dispatch(request, *args, **kwargs)

    def get_initial(self):
        return {'receivers': self.request.GET.getlist('to')}

    def form_valid(self, form):
        message = PrivateMessage(
            sender=self.request.user,
            sender_ip=self.request.META.get('REMOTE_ADDR', ''),
            subject=form.cleaned_data['subject'],
            body=form.cleaned_data['body'],
            )
        message.save()
        for receiver in form.cleaned_data['receivers']:
            MessageHandler(message=message, receiver=receiver).save()
        return HttpResponseRedirect(message.get_absolute_url())



# -*- coding: utf-8 -*-

def unread_messages(request):
    context = {}
    if request.user.is_authenticated():
        context['unread_msgs'] = request.user.inbox.filter(messagehandler__read=False).count()
    return context



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.core.urlresolvers import reverse
from django.db import models
from django.utils.encoding import python_2_unicode_compatible
from django.utils.translation import ugettext as _

from pybb.compat import get_user_model_path
from pybb.models import RenderableItem


class MessageHandler(models.Model):

    message = models.ForeignKey('PrivateMessage')
    receiver = models.ForeignKey(get_user_model_path())
    read = models.BooleanField(default=False)
    deleted = models.BooleanField(default=False)


@python_2_unicode_compatible
class PrivateMessage(RenderableItem):

    class Meta(object):
        verbose_name = _('Private Message')
        verbose_name_plural = _('Private Messages')

    sender = models.ForeignKey(get_user_model_path(),
                               related_name='outbox',
                               verbose_name=_('Sender'))
    sender_ip = models.IPAddressField(_('Sender IP'),
                                      blank=True,
                                      default='0.0.0.0')
    sent = models.DateTimeField(_('Sent'), auto_now_add=True, db_index=True)
    subject = models.CharField(max_length=100, blank=True, default='[No Subject]')
    receivers = models.ManyToManyField(get_user_model_path(),
                                       through='MessageHandler',
                                       related_name='inbox',
                                       verbose_name=_('Recipients'))
    # Expected behaviour when deleting a sent message is that the recipient still has their copy
    sender_deleted = models.BooleanField(default=False)

    def __str__(self):
        return ' '.join([str(self.sender), _('Private Message'), str(self.id)])

    def get_absolute_url(self):
        return reverse('pybb:private_message', kwargs={'pk': self.id})

    def save(self, *args, **kwargs):
      self.render()
      super(PrivateMessage, self).save(*args, **kwargs)
